@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: cconfig.h,v 1.2 2001/11/04 02:53:48 mwedel Exp $";
+ *   "$Id: cconfig.h,v 1.3 2002/03/26 08:15:57 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -29,16 +29,6 @@
  * here can provide nice defaults.
  */
 
-/* Directory to store cached images (only makes a difference if run
- * with the -cache option.)  If not set, then we store in ~/.crossfire/images
- * This should be good enough for most people.  However, if you have a site
- * with many people running the client, you might want to make a standard
- * depository (note that permissions on the repository might need to be
- * pretty wide open.
- */
-
-/*#define IMAGECACHEDIR "/tmp" */
-
 /* This is how often the client checks for X events, as well as how often
  * it performs animations (or will).  This value can be most anything.
  * IT is only configurable because the exact value it needs to be set to
@@ -55,11 +45,6 @@
 /* This is the default port to connect to the server with in string form. */
 #define DEFPORT "13327"
 
-/* Set to default display mode you want (one of Pix_Display, Xpm_Display, or
- * Png_Display).
- */
-#define DISPLAY_MODE Png_Display
-
 /* Set to default server you want the client to connect to.  This can
  * be especially useful if your installing the client binary on a LAN
  * and want people to just be able to run it without options and connect
