@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_client_c =
- *   "$Id: client.c,v 1.17 2003/07/08 12:52:35 tchize Exp $";
- */
+char *rcsid_common_client_c =
+    "$Id: client.c,v 1.23 2004/01/30 18:43:34 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -45,6 +43,7 @@
 #include <client.h>
 #include <external.h>
 #include <errno.h>
+#include <script.h>
 
 /* actually declare the globals */
 
@@ -71,6 +70,7 @@ typedef void (*CmdProc)(unsigned char *, int len);
 struct CmdMapping {
   char *cmdname;
   void (*cmdproc)(unsigned char *, int );
+  enum CmdFormat cmdformat;
 };
 
 
@@ -78,48 +78,48 @@ struct CmdMapping commands[] = {
     /* Order of this table doesn't make a difference.  I tried to sort
      * of cluster the related stuff together.
      */
-    { "map1", Map1Cmd },
-    { "map1a", Map1aCmd },
-    { "map_scroll", (CmdProc)map_scrollCmd },
-    { "magicmap", MagicMapCmd},
-    { "newmap", NewmapCmd },
-    { "mapextended", MapExtendedCmd },
+    { "map1", Map1Cmd, SHORT_ARRAY },
+    { "map1a", Map1aCmd, SHORT_ARRAY },
+    { "map_scroll", (CmdProc)map_scrollCmd, ASCII },
+    { "magicmap", MagicMapCmd, MIXED /* ASCII, then binary */},
+    { "newmap", NewmapCmd, NODATA },
+    { "mapextended", MapExtendedCmd, MIXED /* chars, then SHORT_ARRAY */ },
 
-    { "item1", Item1Cmd },
-    { "item2", Item2Cmd },
-    { "upditem", UpdateItemCmd },
-    { "delitem", DeleteItem },
-    { "delinv",	DeleteInventory },
+    { "item1", Item1Cmd, MIXED /* differing length ints */ },
+    { "item2", Item2Cmd, MIXED },
+    { "upditem", UpdateItemCmd, MIXED },
+    { "delitem", DeleteItem, INT_ARRAY },
+    { "delinv",	DeleteInventory, ASCII },
 
-    { "drawinfo", (CmdProc)DrawInfoCmd },
-    { "stats", StatsCmd },
+    { "drawinfo", (CmdProc)DrawInfoCmd, ASCII },
+    { "stats", StatsCmd, STATS /* array of: int8, (int?s for that stat) */ },
 
-    { "image", ImageCmd },
-    { "image2", Image2Cmd },
-    { "face", FaceCmd},
-    { "face1", Face1Cmd},
-    { "face2", Face2Cmd},
+    { "image", ImageCmd, MIXED /* int, int, PNG */ },
+    { "image2", Image2Cmd, MIXED /* int, int8, int, PNG */ },
+    { "face", FaceCmd, MIXED /* int16, string */},
+    { "face1", Face1Cmd, MIXED /* int16, int32, string */},
+    { "face2", Face2Cmd, MIXED /* int16, int8, int32, string */},
 
 
-    { "sound", SoundCmd},
-    { "anim", AnimCmd},
-    { "smooth", SmoothCmd},
+    { "sound", SoundCmd, MIXED /* int8, int8, int16, int8 */},
+    { "anim", AnimCmd, SHORT_ARRAY},
+    { "smooth", SmoothCmd, SHORT_ARRAY},
 
-    { "player", PlayerCmd },
-    { "comc", CompleteCmd},
+    { "player", PlayerCmd, MIXED /* 3 ints, int8, str */ },
+    { "comc", CompleteCmd, SHORT_INT },
 
-    { "addme_failed", (CmdProc)AddMeFail },
-    { "addme_success", (CmdProc)AddMeSuccess },
-    { "version", (CmdProc)VersionCmd },
-    { "goodbye", (CmdProc)GoodbyeCmd },
-    { "setup", (CmdProc)SetupCmd},
-    { "ExtendedInfoSet", (CmdProc)ExtendedInfoSetCmd},
+    { "addme_failed", (CmdProc)AddMeFail, NODATA },
+    { "addme_success", (CmdProc)AddMeSuccess, NODATA },
+    { "version", (CmdProc)VersionCmd, ASCII },
+    { "goodbye", (CmdProc)GoodbyeCmd, NODATA },
+    { "setup", (CmdProc)SetupCmd, ASCII},
+    { "ExtendedInfoSet", (CmdProc)ExtendedInfoSetCmd, NODATA},
 
-    { "query", (CmdProc)handle_query},
-    { "replyinfo", (CmdProc)ReplyInfoCmd},
+    { "query", (CmdProc)handle_query, ASCII},
+    { "replyinfo", (CmdProc)ReplyInfoCmd, ASCII},
 };
 
-#define NCOMMANDS (sizeof(commands)/sizeof(struct CmdMapping))
+#define NCOMMANDS ((int)(sizeof(commands)/sizeof(struct CmdMapping)))
 
 void DoClient(ClientSocket *csocket)
 {
@@ -145,9 +145,10 @@ void DoClient(ClientSocket *csocket)
 	}
         len = csocket->inbuf.len - (data - csocket->inbuf.buf);
 	/* Terminate the buffer */
-	LOG(0,"Command:%s (%d)\n",csocket->inbuf.buf+2, len);
+	LOG(0,"Common::DoClient","Command:%s (%d)",csocket->inbuf.buf+2, len);
 	for(i=0;i < NCOMMANDS;i++) {
 	    if (strcmp((char*)csocket->inbuf.buf+2,commands[i].cmdname)==0) {
+		    script_watch((char*)csocket->inbuf.buf+2,data,len,commands[i].cmdformat);
 		    commands[i].cmdproc(data,len);
 		    break;
 	    }
@@ -159,6 +160,7 @@ void DoClient(ClientSocket *csocket)
     }
 }
 
+#ifndef WIN32
 #include <netdb.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -166,6 +168,7 @@ void DoClient(ClientSocket *csocket)
 #include <netinet/tcp.h>
 #include <ctype.h>
 #include <arpa/inet.h>
+#endif
 
 /* returns the fd of the connected socket, -1 on failure. */
 
@@ -178,12 +181,13 @@ int init_connection(char *host, int port)
     protox = getprotobyname("tcp");
     if (protox == (struct protoent  *) NULL)
     {
-	fprintf(stderr, "Error getting prorobyname (tcp)\n");
+	LOG (LOG_ERROR,"common::init_connection", "Error getting protobyname (tcp)");
 	return -1;
     }
     fd = socket(PF_INET, SOCK_STREAM, protox->p_proto);
     if (fd==-1) {
-	perror("init_connection:  Error on socket command.\n");
+	    perror("init_connection:  Error on socket command.\n");
+	    LOG (LOG_ERROR,"common::init_connection", "Error on socket command");
 	return -1;
     }
     insock.sin_family = AF_INET;
@@ -194,27 +198,42 @@ int init_connection(char *host, int port)
 	struct hostent *hostbn = gethostbyname(host);
 	if (hostbn == (struct hostent *) NULL)
 	{
-	    fprintf(stderr,"Unknown host: %s\n",host);
+	    LOG (LOG_ERROR,"common::init_connection","Unknown host: %s",host);
 	    return -1;
 	}
 	memcpy(&insock.sin_addr, hostbn->h_addr, hostbn->h_length);
     }
     if (connect(fd,(struct sockaddr *)&insock,sizeof(insock)) == (-1))
     {
-	perror("Can't connect to server");
-	return -1;
+        LOG (LOG_ERROR,"common::init_connection","Can't connect to server");
+	    perror("Can't connect to server");
+	    return -1;
     }
+#ifndef WIN32
     if (fcntl(fd, F_SETFL, O_NDELAY)==-1) {
-	fprintf(stderr,"InitConnection:  Error on fcntl.\n");
+	LOG (LOG_ERROR,"common::init_connection","Error on fcntl.");
     }
+#else
+    {
+		unsigned long tmp = 1;
+		if (ioctlsocket(fd, FIONBIO, &tmp)<0) {
+	    LOG (LOG_ERROR,"common::init_connection","Error on ioctlsocket.");
+        }
+	}
+#endif
 
 #ifdef TCP_NODELAY
     /* turn off nagle algorithm */
     if (use_config[CONFIG_FASTTCP]) {
 	int i=1;
 
+#ifdef WIN32
+	if (setsockopt(fd, SOL_TCP, TCP_NODELAY, ( const char* )&i, sizeof(i)) == -1)
+	    perror("TCP_NODELAY");
+#else
 	if (setsockopt(fd, SOL_TCP, TCP_NODELAY, &i, sizeof(i)) == -1)
 	    perror("TCP_NODELAY");
+#endif
     }
 #endif
 
@@ -223,7 +242,7 @@ int init_connection(char *host, int port)
 
     if (oldbufsize<newbufsize) {
 	if(setsockopt(fd,SOL_SOCKET,SO_RCVBUF, (char*)&newbufsize, sizeof(&newbufsize))) {
-            LOG(1,"InitConnection: setsockopt unable to set output buf size to %d\n", newbufsize);
+            LOG(LOG_WARNING,"InitConnection: setsockopt"," unable to set output buf size to %d", newbufsize);
 	    setsockopt(fd,SOL_SOCKET,SO_RCVBUF, (char*)&oldbufsize, sizeof(&oldbufsize));
 	}
     }
@@ -256,16 +275,20 @@ void negotiate_connection(int sound)
 	tries++;
 	/* If we have't got a response in 10 seconds, bail out */
 	if (tries > 1000) {
+#ifdef WIN32
+	    closesocket(csocket.fd);
+#else
 	    close(csocket.fd);
+#endif
 	    csocket.fd=-1;
 	    return;
 	}
     }
 
     if (csocket.sc_version<1023) {
-	fprintf(stderr,"Server does not support PNG images, yet that is all this client\n");
-	fprintf(stderr,"supports.  Either the server needs to be upgraded, or you need to\n");
-	fprintf(stderr,"downgrade your client.\n");
+	LOG (LOG_WARNING,"common::negotiate_connection","Server does not support PNG images, yet that is all this client");
+	LOG (LOG_WARNING,"common::negotiate_connection","supports.  Either the server needs to be upgraded, or you need to");
+	LOG (LOG_WARNING,"common::negotiate_connection","downgrade your client.");
 	exit(1);
     }
 
