@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_client_h =
- *   "$Id: client.h,v 1.14 2003/10/09 06:40:20 mwedel Exp $";
+ *   "$Id: client.h,v 1.21 2004/01/28 22:16:20 tchize Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -83,6 +83,7 @@ typedef struct Smooths {
 
 extern Smooths smooths[MAXSMOOTH];
 extern int smoothused;
+#define CAN_SMOOTH(__SMOOTHLEVEL) (__SMOOTHLEVEL)
 
 /* ClientSocket could probably hold more of the global values - it could
  * probably hold most all socket/communication related values instead
@@ -210,16 +211,11 @@ typedef struct Player_Struct {
     Input_State input_state;	/* What the input state is */
     char	last_command[MAX_BUF];	/* Last command entered */
     char	input_text[MAX_BUF];	/* keys typed (for long commands) */
-    char	name[40];	/* name and password.  Only used while */
-    char	password[40];	/* logging in. */
-    rangetype	shoottype;	/* What type of range attack player has */
     item	*ranges[range_size];	/* Object that is used for that */
 				/* range type */
     uint8	ready_spell;	/* Index to spell that is readied */
     char	spells[255][40];	/* List of all the spells the */
 				/* player knows */
-    uint8	map_x, map_y;	/* These are offset values.  See object.c */
-				/* for more details */
     Stats	stats;		/* Player stats */
     char	title[MAX_BUF];	/* Title of character */
     char	range[MAX_BUF];	/* Range attack chosen */
@@ -233,8 +229,12 @@ typedef struct Player_Struct {
     uint8	showmagic;	/* If 0, show normal map, otherwise, show
 				 * magic map.
 				 */
-    uint16	mapxres,mapyres;/* resolution to draw on the magic map */
+    uint16	mapxres,mapyres;/* Resolution to draw on the magic map
+                                 * Only used in client-specific code, so it should move there. */
 
+#ifdef WIN32
+    char    name[ 40 ]; /* Player's name, for player-specific key files */
+#endif
 } Client_Player;
 
 
@@ -286,13 +286,53 @@ extern Face_Information face_info;
 extern Client_Player cpl;		/* Player object. */
 extern char *skill_names[MAX_SKILL];
 
+
+typedef enum LogLevel {
+  LOG_DEBUG = 0, LOG_INFO = 1, LOG_WARNING = 2, LOG_ERROR = 3, LOG_CRITICAL = 4
+} LogLevel;
+
+typedef struct LogEntry{
+    char* message;
+    char* origin;
+    LogLevel level;
+    struct LogEntry* next;
+} LogEntry;
+extern LogEntry* LogFirst;
+extern LogEntry* LogLast;
+typedef void (*LogListener)(LogEntry*);
+#define LOG_APPEND(_Entry) if (LogLast) {LogLast->next=_Entry;} else {LogFirst=_Entry;};LogLast=_Entry;_Entry->next=NULL
+#define LOG_NEW_ENTRY (LogEntry*)calloc(1,sizeof(LogEntry))
+#define LOG_SETMESSAGE(_Entry,_msg) if (_Entry->message){free(_Entry->message);};_Entry->message=strdup(_msg)
+#define LOG_SETORIGIN(_Entry,_orig) if (_Entry->origin){free(_Entry->origin);};_Entry->origin=strdup(_orig)
+
+typedef struct{
+    char* name;
+    LogLevel level;
+    int log; /*yes or no to log*/
+}PipeLog;
+#define CHILD_STDIN 1
+#define CHILD_STDOUT 2
+#define CHILD_STDERR 4
+#define CHILD_SILENTFAIL 8
+#define CHILD_TUBE (CHILD_STDIN|CHILD_STDOUT|CHILD_STDERR)
+typedef struct ChildProcess{
+    char* name;
+    int flag;
+    int pid;
+    int tube[3];
+    PipeLog logger[3];
+    struct ChildProcess* next;
+}ChildProcess;
+#define CHILD_PIPEIN(__child) (__child->tube[0])
+#define CHILD_PIPEOUT(__child) (__child->tube[1])
+#define CHILD_PIPEERR(__child) (__child->tube[2])
+
+
 #ifndef CPROTO
 /* We need to declare most of the structs before we can include this */
 #include <proto.h>
 #endif
 
-extern int errno;
-
 /* translation of the STAT_RES names into printable names,
  * in matching order.
  */
@@ -366,6 +406,7 @@ struct MapCell {
     uint8 have_darkness:1;
     uint8 need_resmooth:1;  /*same has need update but for smoothing only*/
     uint8 cleared:1; /* If set, this is a fog cell. */
+    uint8 keephead:1;
 };
 
 
@@ -418,3 +459,4 @@ typedef struct
 extern PlayerPosition pl_pos;
 
 extern struct Map the_map;
+
