@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_image_c =
- *   "$Id: image.c,v 1.8 2003/07/08 12:52:35 tchize Exp $";
- */
+char *rcsid_common_image_c =
+    "$Id: image.c,v 1.16 2004/01/30 22:03:37 ryo_saeba Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -33,7 +31,12 @@
 #include "config.h"
 #include <stdlib.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <unistd.h>
+#else
+#include <io.h>
+#include <direct.h>
+#endif
 #include <ctype.h>
 
 #include "client.h"
@@ -79,7 +82,7 @@ static int load_image(char *filename, uint8 *data, int *len, int *csum)
 	offset = atoi(cp + 1);
 	lp = strchr(cp,':');
 	if (!lp) {
-	    fprintf(stderr,"Corrupt filename - has @ but no :?\n%s\n", filename);
+	    LOG(LOG_ERROR,"common::load_image","Corrupt filename - has '@' but no ':' ?(%s)", filename);
 	    return -1;
 	}
 	length = atoi(lp + 1);
@@ -91,12 +94,16 @@ static int load_image(char *filename, uint8 *data, int *len, int *csum)
 	/* Didn't find a matching entry yet, so make one */
 	if (i == MAX_FACE_SETS) {
 	    if (last == -1) {
-		fprintf(stderr,"fd_cache filled up?  unable to load matching cache entry\n");
+		LOG(LOG_WARNING,"common::load_image","fd_cache filled up?  unable to load matching cache entry");
 		*cp = '@';	/* put @ back in string */
 		return -1;
 	    }
+#ifdef WIN32
+	    if ((fd_cache[last].fd = open(filename, O_RDONLY | O_BINARY))==-1) {
+#else
 	    if ((fd_cache[last].fd = open(filename, O_RDONLY))==-1) {
-		fprintf(stderr,"unable to load listed cache file %s\n",filename);
+#endif
+		LOG(LOG_WARNING,"common::load_image","unable to load listed cache file %s",filename);
 		*cp = '@';	/* put @ back in string */
 		return -1;
 	    }
@@ -104,12 +111,24 @@ static int load_image(char *filename, uint8 *data, int *len, int *csum)
 	    i=last;
 	}
 	lseek(fd_cache[i].fd, offset, SEEK_SET);
+#ifdef WIN32
+	*len = read(fd_cache[i].fd, data, length);
+#else
 	*len = read(fd_cache[i].fd, data, 65535);
+#endif
 	*cp = '@';
     }
     else {
+#ifdef WIN32
+    int length = 0;
+	if ((fd=open(filename, O_RDONLY | O_BINARY))==-1) return -1;
+    length = lseek(fd, 0, SEEK_END);
+    lseek(fd, 0, SEEK_SET);
+    *len=read(fd, data, length);
+#else
 	if ((fd=open(filename, O_RDONLY))==-1) return -1;
 	*len=read(fd, data, 65535);
+#endif
 	close(fd);
     }
 
@@ -175,7 +194,7 @@ static uint32 image_hash_name(char *str, int tablesize) {
     for (p = str; *p!='\0' && *p != '.'; p++) {
         hash ^= (uint32) *p << rot;
         rot += 2;
-        if (rot >= (sizeof(uint32) - sizeof(char)) * 8)
+        if (rot >= (int)(sizeof(uint32) - sizeof(char)) * 8)
             rot = 0;
     }
     return (hash % tablesize);
@@ -200,7 +219,7 @@ static sint32 image_find_hash(char *str)
     /* If the hash table is full, this is bad because we won't be able to
      * add any new entries.
      */
-    fprintf(stderr,"image_find_hash: Hash table is full, increase IMAGE_CACHE size\n");
+    LOG(LOG_WARNING,"common::image_find_hash","Hash table is full, increase IMAGE_CACHE size");
     return -1;
 }
 
@@ -211,7 +230,7 @@ static void image_remove_hash(char *imagename, Cache_Entry *ce)
 
     hash_entry = image_find_hash(imagename);
     if (hash_entry == -1) {
-	fprintf(stderr,"Unable to find cache entry for %s, %s\n", imagename, ce->filename);
+	LOG(LOG_ERROR,"common::image_remove_hash","Unable to find cache entry for %s, %s", imagename, ce->filename);
 	return;
     }
     if (image_cache[hash_entry].cache_entry == ce) {
@@ -223,7 +242,7 @@ static void image_remove_hash(char *imagename, Cache_Entry *ce)
     last = image_cache[hash_entry].cache_entry;
     while (last->next && last->next != ce) last=last->next;
     if (!last->next) {
-	fprintf(stderr,"Unable to find cache entry for %s, %s\n", imagename, ce->filename);
+	LOG(LOG_ERROR,"common::image_rmove_hash","Unable to find cache entry for %s, %s", imagename, ce->filename);
 	return;
     }
     last->next = ce->next;
@@ -268,7 +287,7 @@ static Cache_Entry *image_add_hash(char *imagename, char *filename, uint32 check
 	if (newhash == IMAGE_HASH) newhash=0;
 	/* If the hash table is full, can't do anything */
 	if (newhash == hash) {
-	    fprintf(stderr,"image_find_hash: Hash table is full, increase IMAGE_CACHE size\n");
+	    LOG(LOG_WARNING,"common::image_find_hash","Hash table is full, increase IMAGE_CACHE size");
 	    return NULL;
 	}
     }
@@ -306,7 +325,7 @@ static void image_process_line(char *line, uint32 public)
     if (sscanf(line, "%s %u %s", imagename, &checksum, filename)==3) {
 	image_add_hash(imagename, filename, checksum, public);
     } else {
-	fprintf(stderr,"Did not parse line %s properly?\n", line);
+	LOG(LOG_WARNING,"common::image_process_line","Did not parse line %s properly?", line);
     }
 }
 
@@ -343,7 +362,7 @@ void init_common_cache_data()
 	fd_cache[i].name[0] = '\0';
     }
 }
-    
+
 
 void requestsmooth (int pnum){
     cs_print_string (csocket.fd, "asksmooth %d",pnum);
@@ -439,7 +458,7 @@ void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int fac
 	    sprintf(filename,"%s/.crossfire/crossfire-images/%s",
 		    getenv("HOME"), ce->filename);
 	if (load_image(filename, data, &len, &newsum)==-1) {
-	    fprintf(stderr,"file %s listed in cache file, but unable to load\n", filename);
+	    LOG(LOG_WARNING,"common::finish_face_cmd","file %s listed in cache file, but unable to load", filename);
 	    requestface(pnum, face);
 	    return;
 	}
@@ -449,7 +468,7 @@ void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int fac
 
     if (!(png_tmp = png_to_data(data, len, &nx, &ny))) {
 	/* If the data is bad, remove it if it is in the players private cache */
-	fprintf(stderr,"Got error on png_to_data, image=%s\n",face);
+	LOG(LOG_WARNING,"common::finish_face_cmd","Got error on png_to_data, image=%s",face);
 	if (ce) {
 	    if (!ce->public) unlink(filename);
 	    image_remove_hash(face,ce);
@@ -462,9 +481,11 @@ void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int fac
      * the gui section of the code.
      */
     if (create_and_rescale_image_from_data(ce, pnum, png_tmp,nx, ny)) {
-	fprintf(stderr,"Got error on create_and_rescale_image_from_data, file=%s\n",filename);
+	LOG(LOG_WARNING,"common::finish_face_cmd","Got error on create_and_rescale_image_from_data, file=%s",filename);
 	requestface(pnum, face);
     }
+    if (png_tmp != NULL)
+        free(png_tmp);
 }
 
 
@@ -508,7 +529,7 @@ void FaceCmd(unsigned char *data,  int len)
      * structures may not be initialized.
      */
     if (!use_config[CONFIG_CACHE]) {
-	fprintf(stderr,"Received a 'face' command when we are not caching\n");
+	LOG(LOG_WARNING,"common::FaceCmd","Received a 'face' command when we are not caching");
 	return;
     }
     pnum = GetShort_String(data);
@@ -529,7 +550,7 @@ void Face1Cmd(unsigned char *data,  int len)
      * structures may not be initialized.
      */
     if (!use_config[CONFIG_CACHE]) {
-	fprintf(stderr,"Received a 'face' command when we are not caching\n");
+	LOG(LOG_WARNING,"common::Face1Cmd","Received a 'face' command when we are not caching");
 	return;
     }
     pnum = GetShort_String(data);
@@ -551,7 +572,7 @@ void Face2Cmd(uint8 *data,  int len)
      * structures may not be initialized.
      */
     if (!use_config[CONFIG_CACHE]) {
-	fprintf(stderr,"Received a 'face' command when we are not caching\n");
+	LOG(LOG_WARNING,"common::Face2Cmd","Received a 'face' command when we are not caching");
 	return;
     }
     pnum = GetShort_String(data);
@@ -570,7 +591,7 @@ void ImageCmd(uint8 *data,  int len)
     pnum = GetInt_String(data);
     plen = GetInt_String(data+4);
     if (len<8 || (len-8)!=plen) {
-	fprintf(stderr,"ImageCmd: Lengths don't compare (%d,%d)\n",
+	LOG(LOG_WARNING,"common::ImageCmd","Lengths don't compare (%d,%d)",
 		(len-8),plen);
 	return;
     }
@@ -587,7 +608,7 @@ void Image2Cmd(uint8 *data,  int len)
     setnum = data[4];
     plen = GetInt_String(data+5);
     if (len<9 || (len-9)!=plen) {
-	fprintf(stderr,"Image2Cmd: Lengths don't compare (%d,%d)\n",
+	LOG(LOG_WARNING,"common::Image2Cmd","Lengths don't compare (%d,%d)",
 		(len-9),plen);
 	return;
     }
@@ -609,17 +630,25 @@ void display_newpng(long face,uint8 *buf,long buflen, int setnum)
 
     if (use_config[CONFIG_CACHE]) {
 	if (facetoname[face]==NULL) {
-	    fprintf(stderr,"Caching images, but name for %ld not set\n", face);
+	    LOG(LOG_WARNING,"common::display_newpng","Caching images, but name for %ld not set", face);
 	}
 	/* Make necessary leading directories */
 	sprintf(filename, "%s/.crossfire/crossfire-images",getenv("HOME"));
 	if (access(filename, R_OK | W_OK | X_OK)== -1) 
+#ifdef WIN32
+	    mkdir(filename);
+#else
 	    mkdir(filename, 0755);
+#endif
 
 	sprintf(filename, "%s/.crossfire/crossfire-images/%c%c", getenv("HOME"),
 		 facetoname[face][0], facetoname[face][1]);
 	if (access(filename, R_OK | W_OK | X_OK)== -1) 
+#ifdef WIN32
+	    mkdir(filename);
+#else
 	    mkdir(filename,0755);
+#endif
 
 	/* If setnum is valid, and we have faceset information for it,
 	 * put that prefix in.  This will make it easier later on to 
@@ -645,8 +674,12 @@ void display_newpng(long face,uint8 *buf,long buflen, int setnum)
 		    facetoname[face][1], basename, setnum);
 	} while (access(filename, F_OK)==-0);
 
+#ifdef WIN32
+	if ((tmpfile = fopen(filename,"wb"))==NULL) {
+#else
 	if ((tmpfile = fopen(filename,"w"))==NULL) {
-	    fprintf(stderr,"Can not open %s for writing\n", filename);
+#endif
+	    LOG(LOG_WARNING,"common::display_newpng","Can not open %s for writing", filename);
 	}
 	else {
 	    /* found a file we can write to */
@@ -654,12 +687,12 @@ void display_newpng(long face,uint8 *buf,long buflen, int setnum)
 	    fwrite(buf, buflen, 1, tmpfile);
 	    fclose(tmpfile);
 	    csum=0;
-	    for (i=0; i<buflen; i++) {
+	    for (i=0; (int)i<buflen; i++) {
 		ROTATE_RIGHT(csum);
 		csum += buf[i];
 		csum &= 0xffffffff;
 	    }
-	    sprintf(filename, "%c%c/%s.%d", facetoname[face][0], facetoname[face][1], 
+	    sprintf(filename, "%c%c/%s.%d", facetoname[face][0], facetoname[face][1],
 		    basename, setnum);
 	    ce = image_add_hash(facetoname[face], filename,  csum, 0);
 
@@ -671,7 +704,7 @@ void display_newpng(long face,uint8 *buf,long buflen, int setnum)
 	     */
 	    sprintf(filename,"%s/.crossfire/crossfire-images/bmaps.client", getenv("HOME"));
 	    if ((tmpfile=fopen(filename, "a"))==NULL) {
-		fprintf(stderr,"Can not open %s for appending\n", filename);
+		LOG(LOG_WARNING,"common::display_newpng","Can not open %s for appending", filename);
 	    }
 	    else {
 		fprintf(tmpfile, "%s %u %c%c/%s.%d\n",
@@ -691,6 +724,8 @@ void display_newpng(long face,uint8 *buf,long buflen, int setnum)
 	    facetoname[face]=NULL;
 	}
     }
+    if (pngtmp != NULL)
+        free(pngtmp);
 }
 
 
@@ -702,7 +737,7 @@ void display_newpng(long face,uint8 *buf,long buflen, int setnum)
  * operations with safety.
  * In each block, we find the newline - if we find one, we presume
  * the data is good, and update the face_info accordingly.
- * if we don't find a newline, we return.  
+ * if we don't find a newline, we return.
  */
 void get_image_info(char *data, int len)
 {
@@ -734,11 +769,11 @@ void get_image_info(char *data, int len)
             if (!(cps[i] = strtok(NULL, ":"))) badline=1;
 	}
         if (badline) {
-            fprintf(stderr,"get_image_info - bad data, ignoring line:\n  %s", lp);
+            LOG(LOG_WARNING,"common::get_image_info","bad data, ignoring line:/%s/", lp);
 	} else {
             onset = atoi(cps[0]);
             if (onset >=MAX_FACE_SETS) {
-		fprintf(stderr,"get_image_info: setnum is too high: %d > %d\n",
+		LOG(LOG_WARNING,"common::get_image_info","setnum is too high: %d > %d",
                     onset, MAX_FACE_SETS);
 	    }
             face_info.facesets[onset].prefix = strdup_local(cps[1]);
@@ -828,7 +863,7 @@ void get_image_sums(uint8 *data, int len)
 	 */
 	finish_face_cmd(imagenum, checksum, 1, cp, faceset);
 	if (imagenum > stop) 
-	    fprintf(stderr,"Received an image beyond our range? %d > %d\n", imagenum, stop);
+	    LOG(LOG_WARNING,"common::get_image_sums","Received an image beyond our range? %d > %d", imagenum, stop);
 	cp += slen;
     }
 }
