@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.14 2003/10/09 06:40:20 mwedel Exp $";
- */
+char *rcsid_common_init_c =
+    "$Id: init.c,v 1.19 2004/02/01 09:31:57 ryo_saeba Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -57,21 +55,21 @@ void VersionCmd(char *data, int len)
     csocket.cs_version = atoi(data);
     /* set sc_version in case it is an old server supplying only one version */
     csocket.sc_version = csocket.cs_version;
-    if (csocket.cs_version != atoi(data)) {
-	fprintf(stderr,"Differing C->S version numbers (%d,%d)\n",
-	   VERSION_CS,csocket.cs_version);
+    if (csocket.cs_version != VERSION_CS) {
+        LOG(LOG_WARNING,"common::VersionCmd","Differing C->S version numbers (%d,%d)",
+            VERSION_CS,csocket.cs_version);
 /*	exit(1);*/
     }
     cp = strchr(data,' ');
     if (!cp) return;
     csocket.sc_version = atoi(cp);
     if (csocket.sc_version != VERSION_SC) {
-	fprintf(stderr,"Differing S->C version numbers (%d,%d)\n",
-	   VERSION_SC,csocket.sc_version);
+    LOG(LOG_WARNING,"common::VersionCmd","Differing S->C version numbers (%d,%d)",
+        VERSION_SC,csocket.sc_version);
     }
     cp = strchr(cp+1, ' ');
     if (cp)
-	fprintf(stderr,"Playing on server type %s\n", cp);
+	LOG(LOG_INFO,"common::VersionCmd","Playing on server type %s", cp);
 }
 
 void SendVersion(ClientSocket csock)
@@ -128,9 +126,6 @@ void init_client_vars()
 	skill_names[i] = NULL;
     }
 
-    cpl.map_x=0;
-    cpl.map_y=0;
-
     cpl.ob = player_item();
     cpl.below = map_item();
     cpl.magicmap=NULL;
@@ -197,6 +192,14 @@ void init_client_vars()
     for (i=0; i<CONFIG_NUMS; i++) 
 	use_config[i] = want_config[i];
 
+#ifdef WIN32
+    /* If HOME is not set, let's set it to . to avoid things like (null)/.crossfire paths */
+    if ( !getenv( "HOME" ) )
+        {
+        LOG( LOG_INFO, "common::init.c", "init_client_vars: HOME not set, setting it to .\n" );
+        putenv( "HOME=." );
+        }
+#endif
 }
 
 /* This is basically called each time a new player logs
@@ -237,9 +240,6 @@ void reset_client_vars()
     for (i=0; i<range_size; i++)
 	cpl.ranges[i]=NULL;
 
-    cpl.map_x=0;
-    cpl.map_y=0;
-
     cpl.magicmap=NULL;
     cpl.showmagic=0;
 
