@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.6 2003/10/10 06:18:52 mwedel Exp $";
- */
+char *rcsid_common_item_c =
+    "$Id: item.c,v 1.10 2004/01/30 18:43:34 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -30,6 +28,7 @@
 #include <item.h>
 #include <newclient.h>
 #include <external.h>
+#include <script.h>
 
 static item *free_items;	/* the list of free (unused) items */
 static item *player, *map;	/* these lists contains rest of items */
@@ -77,7 +76,7 @@ uint8 get_type_from_name(const char *name)
 	    pos++;
 	}
     }
-    fprintf(stderr,"Could not find match for %s\n", name);
+    LOG(LOG_WARNING,"common::get_type_from_name","Could not find match for %s", name);
     return 255;
 }
 
@@ -366,6 +365,8 @@ void remove_item (item *op)
  */
 void remove_item_inventory (item *op)
 {
+    if ( !op )
+        return;
     op->inv_updated = 1;
     while (op->inv)
 	remove_item (op->inv);
@@ -541,6 +542,8 @@ void toggle_locked (item *op)
     if (op->env->tag == 0)
 	return;	/* if item is on the ground, don't lock it */
 
+    sprintf(buf,"lock %c %d",!op->locked,op->tag);
+    script_monitor_str(buf);
     SockList_Init(&sl, buf);
     SockList_AddString(&sl, "lock "); 
     SockList_AddChar(&sl, !op->locked);
@@ -555,6 +558,8 @@ void send_mark_obj (item *op) {
     if (op->env->tag == 0)
 	return;	/* if item is on the ground, don't mark it */
 
+    sprintf(buf,"mark %d",op->tag);
+    script_monitor_str(buf);
     SockList_Init(&sl, buf);
     SockList_AddString(&sl, "mark ");
     SockList_AddInt(&sl, op->tag);
