@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_metaserver_c =
- *   "$Id: metaserver.c,v 1.3 2002/04/03 07:46:52 mwedel Exp $";
- */
+char *rcsid_common_metaserver_c =
+    "$Id: metaserver.c,v 1.10 2004/04/04 10:07:46 ryo_saeba Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -29,12 +27,14 @@
  * to the server when requested.
  */
 
+#ifndef WIN32
 #include <netdb.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <ctype.h>
 #include <arpa/inet.h>
+#endif /* WIN32 */
 #include <stdio.h>
 
 #include <client.h>
@@ -67,6 +67,78 @@ int meta_numservers=0;
 
 static int meta_sort(Meta_Info *m1, Meta_Info *m2) { return strcasecmp(m1->hostname, m2->hostname); }
 
+#ifdef WIN32
+/* Need script.h for script_killall */
+#include <script.h>
+
+/* This gets input from a socket, and returns it one line at a time.
+ */
+/* This is a Windows-specific function, since you can't use fgets under Win32 */
+char* get_line_from_sock(char* s, size_t n, int fd)
+{
+    static long charsleft = 0;
+    static char inbuf[MS_LARGE_BUF*4];
+    char* cp;
+	int ct;
+
+    if (!s) return s;
+    if (n != MS_LARGE_BUF*4-1)
+    {
+        LOG(LOG_CRITICAL,"common::get_line_from_sock","Serious program logic error in get_line_from_sock().");
+        exit(-1);
+    }
+
+    if ( (charsleft > (MS_LARGE_BUF*4-3)) && (strchr(inbuf,'\n')==NULL) )
+    {
+        draw_info("Metaserver returned an overly long line.", NDI_BLACK);
+        return NULL;
+    }
+
+	/* If there is no line in the buffer */
+    while ( (charsleft==0) || ((cp=strchr(inbuf,'\n'))==NULL) )
+    {
+        FD_SET fdset = {1,fd};
+        TIMEVAL tv = {3,0}; /* 3 second timeout on reads */
+        int nlen;
+        if ( select(0,&fdset,NULL,NULL,&tv) == 0 )
+        {
+            draw_info("Metaserver timed out.", NDI_BLACK);
+            return NULL;
+        }
+        else
+        {
+            nlen = recv(fd,inbuf+charsleft-1,MS_LARGE_BUF*4-1-charsleft,0);
+            if ((nlen == SOCKET_ERROR) || (nlen <= 0)) /* Probably EOF */
+                return NULL;
+			else
+				charsleft += nlen;
+        }
+    }
+
+    /* OK, inbuf contains a null terminated string with at least one \n
+     * Copy the string up to the \n to s, and then move the rest of the
+     * inbuf string to the beginning of the buffer.  And finally, set
+     * charsleft to the number of characters left in inbuf, or 0.
+     * Oh, and cp contains the location of the \n.
+     */
+
+    memcpy(s, inbuf, cp-inbuf+1); /* Extract the line, including the \n. */
+	s[cp-inbuf+1] = 0; /* null terminate it */
+
+    /* Copy cp to inbuf up to the \0, (skipping the \n) */
+    ct = 0;
+    while (cp[++ct] != 0) {
+		inbuf[ct-1] = cp[ct];
+	}
+	inbuf[ct-1] = 0;
+    charsleft = ct;    /* And keep track of how many characters are left. */
+
+    return s;
+}
+
+#endif /* Win32 */
+
+
 /* This contacts the metaserver and gets the list of servers.  returns 0
  * on success, 1 on failure.  Errors will get dumped to stderr,
  * so most errors should be reasonably clear.
@@ -79,7 +151,9 @@ int metaserver_get_info(char *metaserver, int meta_port)
     struct protoent *protox;
     int fd;
     struct sockaddr_in insock;
+#ifndef WIN32
     FILE *fp;
+#endif
     char    inbuf[MS_LARGE_BUF*4];
 
     if (!metaserver_on) {
@@ -90,13 +164,13 @@ int metaserver_get_info(char *metaserver, int meta_port)
     protox = getprotobyname("tcp");
     if (protox == (struct protoent  *) NULL)
     {
-        fprintf(stderr, "Error getting protobyname (tcp)\n");
+        LOG(LOG_WARNING,"common::metaserver_get_info","Error getting protobyname (tcp)");
         return 1;
     }
     fd = socket(PF_INET, SOCK_STREAM, protox->p_proto);
     if (fd==-1) {
         perror("get_metaserver_info:  Error on socket command.\n");
-	return 1;
+	    return 1;
     }
     insock.sin_family = AF_INET;
     insock.sin_port = htons((unsigned short)meta_port);
@@ -106,16 +180,19 @@ int metaserver_get_info(char *metaserver, int meta_port)
         struct hostent *hostbn = gethostbyname(metaserver);
         if (hostbn == (struct hostent *) NULL)
         {
-            fprintf(stderr,"Unknown metaserver hostname: %s\n",metaserver);
-	    return 1;
-	}
+            LOG(LOG_WARNING,"common::metaserver_get_info","Unknown metaserver hostname: %s",metaserver);
+	        return 1;
+	    }
         memcpy(&insock.sin_addr, hostbn->h_addr, hostbn->h_length);
     }
     if (connect(fd,(struct sockaddr *)&insock,sizeof(insock)) == (-1))
     {
-        perror("Can't connect to server");
-	return 1;
+        perror("Can't connect to metaserver");
+        draw_info("\nCan't connect to metaserver.",NDI_BLACK);
+    	return 1;
     }
+
+#ifndef WIN32 /* Windows doesn't support this */
     /* Turn this into a file handle - this will break it on newlines
      * for us, which makes our processing much easier - it basically
      * means one line/server
@@ -124,6 +201,8 @@ int metaserver_get_info(char *metaserver, int meta_port)
 	perror("fdopen failed.");
 	return 1;
     }
+#endif
+
     /* Don't reset this until here - that way if we successfully got
      * a list before, we can still use it.
      */
@@ -136,11 +215,15 @@ int metaserver_get_info(char *metaserver, int meta_port)
      * this is so if there is a corrupt entry, it gets displayed as
      * originally received from the server.
      */
+#ifndef WIN32 /* Windows doesn't support this */
     while (fgets(inbuf, MS_LARGE_BUF*4-1, fp)!=NULL) {
+#else
+    while (get_line_from_sock(inbuf, MS_LARGE_BUF*4-1, fd)!=NULL) {
+#endif
 	char *cp,*cp1;
 
 	if ((cp=strchr(inbuf,'|'))==NULL) {
-	    fprintf(stderr,"Corrupt line from server: %s\n", inbuf);
+	    LOG(LOG_WARNING,"common::metaserver_get_info","Corrupt line from server: %s", inbuf);
 	    break;
 	}
 	*cp=0;
@@ -151,13 +234,13 @@ int metaserver_get_info(char *metaserver, int meta_port)
 	meta_servers[meta_numservers].idle_time = atoi(cp);
 
 	if ((cp1=strchr(cp,'|'))==NULL) {
-	    fprintf(stderr,"Corrupt line from server: %s\n", inbuf);
+	    LOG(LOG_WARNING,"common::metaserver_get_info","Corrupt line from server: %s", inbuf);
 	    break;
 	}
 	*cp1=0;
 
 	if ((cp=strchr(cp1+1,'|'))==NULL) {
-	    fprintf(stderr,"Corrupt line from server: %s\n", inbuf);
+	    LOG(LOG_WARNING,"common::metaserver_get_info","Corrupt line from server: %s", inbuf);
 	    break;
 	}
 	*cp=0;
@@ -171,13 +254,13 @@ int metaserver_get_info(char *metaserver, int meta_port)
 	meta_servers[meta_numservers].players = atoi(cp);
 
 	if ((cp1=strchr(cp,'|'))==NULL) {
-	    fprintf(stderr,"Corrupt line from server: %s\n", inbuf);
+	    LOG(LOG_WARNING,"common::metaserver_get_info","Corrupt line from server: %s", inbuf);
 	    break;
 	}
 	*cp1=0;
 
 	if ((cp=strchr(cp1+1,'|'))==NULL) {
-	    fprintf(stderr,"Corrupt line from server: %s\n", inbuf);
+	    LOG(LOG_WARNING,"common::metaserver_get_info","Corrupt line from server: %s", inbuf);
 	    break;
 	}
 	*cp=0;
@@ -189,7 +272,7 @@ int metaserver_get_info(char *metaserver, int meta_port)
 	*cp++='|';  /* cp now points to comment */
 
 	if ((cp1=strchr(cp,'\n'))==NULL) {
-	    fprintf(stderr,"Corrupt line from server: %s\n", inbuf);
+	    LOG(LOG_WARNING,"common::metaserver_get_info","Corrupt line from server: %s", inbuf);
 	    break;
 	}
 	*cp1=0;
@@ -199,8 +282,12 @@ int metaserver_get_info(char *metaserver, int meta_port)
 
 	meta_numservers++;
     }
+#ifdef WIN32
+    closesocket( fd );
+#else
     fclose(fp);
-    qsort(meta_servers, meta_numservers, sizeof(Meta_Info), (int (*)())meta_sort);
+#endif
+    qsort(meta_servers, meta_numservers, sizeof(Meta_Info), (int (*)(const void*, const void*))meta_sort);
     return 0;
 }
 
@@ -254,7 +341,13 @@ int metaserver_select(char *sel)
     /* Special case - player really entered a 0, so exit the
      * program.
      */
-    if (num==0 && sel[0]=='0') exit(0);
+    if (num==0 && sel[0]=='0')
+        {
+#ifdef WIN32
+    	script_killall();
+#endif
+        exit(0);
+        }
 
     /* if the entry is not a number (selection from the list), 
      * or is a selection but also has a dot (suggesting 
@@ -287,7 +380,7 @@ int metaserver_select(char *sel)
     draw_info(buf,NDI_BLACK);
     csocket.fd=init_connection(server_ip, use_config[CONFIG_PORT]);
     if (csocket.fd==-1) {
-	draw_info("Unable to connect to server.", NDI_RED);
+	draw_info("Unable to connect to server.", NDI_BLACK);
 	return 1;
     }
     return 0;
