@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_newsocket_c =
- *   "$Id: newsocket.c,v 1.6 2003/06/19 12:23:39 tchize Exp $";
- */
+char *rcsid_common_newsocket_c =
+    "$Id: newsocket.c,v 1.12 2004/01/30 18:43:34 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -36,11 +34,12 @@
 
 #include <client.h>
 #include <newclient.h>
+#include <script.h>
 
 /* The LOG function is normally part of the libcross.  If client compile,
  * we need to supply something since some of the common code calls this.
  */
-void LOG (int logLevel, char *format, ...)
+/*void LOG (int logLevel, char *format, ...)
 {
 #ifdef DEBUG
     va_list ap;
@@ -48,7 +47,7 @@ void LOG (int logLevel, char *format, ...)
     vfprintf(stderr, format, ap);
     va_end(ap);
 #endif
-}
+}*/
 
 
 /* We don't care what these values are in the client, since
@@ -69,15 +68,20 @@ static int write_socket(int fd, unsigned char *buf, int len)
     /* If we manage to write more than we wanted, take it as a bonus */
     while (len>0) {
 	do {
+#ifndef WIN32
 	    amt=write(fd, pos, len);
 	} while ((amt<0) && (errno==EINTR));
+#else
+	    amt=send(fd, pos, len, 0);
+	} while ((amt<0) && (WSAGetLastError()==EINTR));
+#endif
 
 	if (amt < 0) { /* We got an error */
-	    LOG(llevError,"New socket (fd=%d) write failed.\n", fd);
+	    LOG(llevError,"write_socket","New socket (fd=%d) write failed.\n", fd);
 	    return -1;
 	}
 	if (amt==0) {
-	    LOG(llevError,"Write_To_Socket: No data written out.\n");
+	    LOG(llevError,"write_socket","Write_To_Socket: No data written out.\n");
 	}
 	len -= amt;
 	pos += amt;
@@ -147,9 +151,15 @@ int GetInt_String(unsigned char *data)
 /* 64 bit version of the above */
 sint64 GetInt64_String(unsigned char *data)
 {
-    return (((uint64)data[0]<<56) + ((uint64)data[1]<<48) + 
-	    ((uint64)data[2]<<40) + ((uint64)data[3]<<32) +
-	    (data[4]<<24) + (data[5]<<16) + (data[6]<<8) + data[7]);
+#ifdef WIN32
+    return (((sint64)data[0]<<56) + ((sint64)data[1]<<48) + 
+	    ((sint64)data[2]<<40) + ((sint64)data[3]<<32) +
+	    ((sint64)data[4]<<24) + ((sint64)data[5]<<16) + ((sint64)data[6]<<8) + (sint64)data[7]);
+#else
+     return (((uint64)data[0]<<56) + ((uint64)data[1]<<48) +
+ 	    ((uint64)data[2]<<40) + ((uint64)data[3]<<32) +
+ 	    ((uint64)data[4]<<24) + (data[5]<<16) + (data[6]<<8) + data[7]);
+#endif
 }
 
 short GetShort_String(unsigned char *data) {
@@ -166,20 +176,28 @@ short GetShort_String(unsigned char *data) {
 int SockList_ReadPacket(int fd, SockList *sl, int len)
 {
     int stat,toread,readsome=0;
-    extern int errno;
 
     /* We already have a partial packet */
     if (sl->len<2) {
 	do {
+#ifndef WIN32
 	    stat=read(fd, sl->buf + sl->len, 2-sl->len);
 	} while ((stat==-1) && (errno==EINTR));
+#else
+	    stat=recv(fd, sl->buf + sl->len, 2-sl->len, 0);
+	} while ((stat==-1) && (WSAGetLastError()==EINTR));
+#endif
 	if (stat<0) {
 	    /* In non blocking mode, EAGAIN is set when there is no
 	     * data available.
 	     */
+#ifndef WIN32
 	    if (errno!=EAGAIN && errno!=EWOULDBLOCK) {
+#else
+	    if (WSAGetLastError()!=EAGAIN && WSAGetLastError()!=WSAEWOULDBLOCK) {
+#endif
 		perror("ReadPacket got an error.");
-		LOG(llevDebug,"ReadPacket got error %d, returning -1",errno);
+		LOG(llevDebug,"SockList_ReadPacket","ReadPacket got error %d, returning -1",errno);
 		return -1;
 	    }
 	    return 0;	/*Error */
@@ -198,18 +216,27 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
      */
     toread = 2+(sl->buf[0] << 8) + sl->buf[1] - sl->len;
     if ((toread + sl->len) > len) {
-	LOG(llevError,"SockList_ReadPacket: Want to read more bytes than will fit in buffer.\n");
+	LOG(llevError,"SockList_ReadPacket","Want to read more bytes than will fit in buffer.\n");
 	/* return error so the socket is closed */
 	return -1;
     }
     do {
 	do {
+#ifndef WIN32
 	    stat = read(fd, sl->buf+ sl->len, toread);
 	} while ((stat<0) && (errno==EINTR));
+#else
+	    stat = recv(fd, sl->buf+ sl->len, toread, 0);
+	} while ((stat<0) && (WSAGetLastError()==EINTR));
+#endif
 	if (stat<0) {
+#ifndef WIN32
 	    if (errno!=EAGAIN && errno!=EWOULDBLOCK) {
+#else
+	    if (WSAGetLastError()!=EAGAIN && WSAGetLastError()!=WSAEWOULDBLOCK) {
+#endif
 		perror("ReadPacket got an error.");
-		LOG(llevDebug,"ReadPacket got error %d, returning 0",errno);
+		LOG(llevDebug,"SockList_ReadPacket","ReadPacket got error %d, returning 0",errno);
 	    }
 	    return 0;	/*Error */
 	}
@@ -222,7 +249,7 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
 	toread -= stat;
 	if (toread==0) return 1;
 	if (toread < 0) {
-	    LOG(llevError,"SockList_ReadPacket: Read more bytes than desired.");
+	    LOG(llevError,"SockList_ReadPacket","SockList_ReadPacket: Read more bytes than desired.");
 	    return 1;
 	}
     } while (toread>0);
@@ -243,5 +270,7 @@ int cs_print_string(int fd, char *str, ...)
     sl.len += vsprintf(sl.buf + sl.len, str, args);
     va_end(args);
 
+    script_monitor_str(sl.buf);
+
     return SockList_Send(&sl, fd);
 }
