@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_image_c =
- *   "$Id: image.c,v 1.13 2003/07/08 12:52:35 tchize Exp $";
- */
+char *rcsid_gtk_image_c =
+    "$Id: image.c,v 1.17 2004/01/30 18:43:34 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -34,12 +32,18 @@
 #include "config.h"
 #include <stdlib.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 #include <png.h>
 
 /* Pick up the gtk headers we need */
 #include <gtk/gtk.h>
+#ifndef WIN32
 #include <gdk/gdkx.h>
+#else
+#include <gdk/gdkwin32.h>
+#endif
 #include <gdk/gdkkeysyms.h>
 
 #ifdef HAVE_SDL
@@ -60,6 +64,8 @@ struct {
     uint32  width, height;
 } private_cache[MAXPIXMAPNUM];
 
+#define BPP 4
+
 /* This holds the name we recieve with the 'face' command so we know what
  * to save it as when we actually get the face.
  */
@@ -118,7 +124,7 @@ static void create_icon_image(uint8 *data, PixmapInfo *pi, int pixmap_num)
     if (rgba_to_gdkpixmap(gtkwin_root->window, data, pi->icon_width, pi->icon_height,
 		(GdkPixmap**)&pi->icon_image, (GdkBitmap**)&pi->icon_mask,
 		gtk_widget_get_colormap(gtkwin_root)))
-		    fprintf(stderr,"Unable to create scaled image, dest num = %d\n", pixmap_num);
+		    LOG (LOG_ERROR,"gtk::create_icon_image","Unable to create scaled image, dest num = %d\n", pixmap_num);
 }
 
 /* These little helper functions just make the code below much more readable */
@@ -292,7 +298,15 @@ int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *r
     } else {
 	pi->map_width = width;
 	pi->map_height = height;
-	create_map_image(rgba_data, pi);
+	/* if using SDL mode, a copy of the rgba data needs to be
+	 * stored away. 
+	 */
+	if (use_config[CONFIG_SDL]) {
+	    png_tmp = malloc(width * height * BPP);
+	    memcpy(png_tmp, rgba_data, width * height * BPP);
+	} else
+	    png_tmp = rgba_data;
+	create_map_image(png_tmp, pi);
     }
     /* Not ideal, but basically, if it is missing the map or icon image, presume
      * something failed.
