@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_keys_c =
- *   "$Id: keys.c,v 1.6 2003/01/23 08:17:09 mwedel Exp $";
- */
+char *rcsid_gtk_keys_c =
+    "$Id: keys.c,v 1.15 2004/03/04 22:31:25 ryo_saeba Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -32,13 +30,22 @@
 #include "config.h"
 #include <stdlib.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 
 /*#include <X11/keysym.h>*/
 
 /* Pick up the gtk headers we need */
 #include <gtk/gtk.h>
+#ifndef WIN32
 #include <gdk/gdkx.h>
+#else
+#include <gdk/gdkwin32.h>
+#define NoSymbol 0L /* Special KeySym */
+typedef int KeyCode; /* Undefined type */
+#include <io.h> /* access( ) */
+#endif
 #include <gdk/gdkkeysyms.h>
 
 #include "client-types.h"
@@ -149,9 +156,11 @@ static void parse_keybind_line(char *buf, int line, int standard)
     uint32 keysym;
     int flags;
 
+    cp = NULL; /* There may be a rare error case when cp is used uninitialized. So let's be safe */
+
     if (buf[0]=='#' || buf[0]=='\n') return;
     if ((cpnext = strchr(buf,' '))==NULL) {
-	fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line,buf);
+	LOG(LOG_WARNING,"gtk::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line,buf);
 	return;
     }
     /* Special keybinding line */
@@ -160,20 +169,20 @@ static void parse_keybind_line(char *buf, int line, int standard)
 	while (*cpnext == ' ') ++cpnext;
 	cp = strchr(cpnext, ' ');
 	if (!cp) {
-	    fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line,buf);
+	    LOG(LOG_WARNING,"gtk::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line,buf);
 	    return;
 	}
 	*cp++ = 0;  /* Null terminate it */
 	cp1 = strchr(cp, ' ');
 	if (!cp1) {
-	    fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line,buf);
+	    LOG(LOG_WARNING,"gtk::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line,buf);
 	    return;
 	}
 	*cp1 ++ = 0;/* Null terminate it */
 	keysym = gdk_keyval_from_name(cp);
 	/* As of now, all these keys must have keysyms */
 	if (keysym == 0) {
-	    fprintf(stderr,"Could not convert %s into keysym\n", cp);
+	    LOG(LOG_WARNING,"gtk::parse_keybind_line","Could not convert %s into keysym", cp);
 	    return;
 	}
 	if (!strcmp(cpnext,"commandkey")) {
@@ -215,19 +224,19 @@ static void parse_keybind_line(char *buf, int line, int standard)
 	*cpnext++ = '\0';
 	keysym = gdk_keyval_from_name(buf);
 	if (!keysym) {
-	    fprintf(stderr,"Unable to convert line %d (%s) into keysym\n", line, cp);
+	    LOG(LOG_WARNING,"gtk::parse_keybind_line","Unable to convert line %d (%s) into keysym", line, cp);
 	    return;
 	}
 	cp = cpnext;
 	if ((cpnext = strchr(cp,' '))==NULL) {
-	    fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line, cp);
+	    LOG(LOG_WARNING,"gtk::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line, cp);
 	    return;
 	}
 	*cpnext++ = '\0';
 
 	cp = cpnext;
 	if ((cpnext = strchr(cp,' '))==NULL) {
-	    fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line, cp);
+	    LOG(LOG_WARNING,"gtk::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line, cp);
 	    return;
 	}
 	*cpnext++ = '\0';
@@ -253,7 +262,7 @@ static void parse_keybind_line(char *buf, int line, int standard)
 		    flags |= KEYF_STANDARD;
 		    break;
 		default:
-		    fprintf(stderr,"Warning:  Unknown flag (%c) line %d in key binding file\n",
+		    LOG(LOG_WARNING,"gtk::parse_keybind_line","Unknown flag (%c) line %d in key binding file",
 			    *cp, line);
 	    }
 	    cp++;
@@ -261,6 +270,10 @@ static void parse_keybind_line(char *buf, int line, int standard)
 
 	/* Rest of the line is the actual command.  Lets kill the newline */
 	cpnext[strlen(cpnext)-1]='\0';
+    if (strlen(cpnext)>(sizeof(bind_buf)-1)){
+        cpnext[sizeof(bind_buf)-1]='\0';
+        LOG(LOG_WARNING,"gtk::parse_keybind_line","Had to truncate a too long command");
+    }
 	insert_key(keysym, flags | standard, cpnext);
     } /* else if not special binding line */
 }
@@ -325,9 +338,24 @@ void init_keys()
      * bind everything.  Probably not a good idea, however.
      */
 
+#ifdef WIN32
+    /* For Windows, use player name if defined for key file */
+    if ( strlen( cpl.name ) )
+        {
+        sprintf( buf, "%s/.crossfire/%s.keys", getenv( "HOME" ), cpl.name );
+        if ( access( buf, 0 ) == -1 )
+            {
+            /* Client key file not found, reverting to default file */
+            sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
+            }
+        }
+    else
+        sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
+#else
     sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
+#endif
     if ((fp=fopen(buf,"r"))==NULL) {
-	fprintf(stderr,"Could not open ~/.crossfire/keys, trying to load global bindings\n");
+	LOG(LOG_INFO,"gtk::init_keys","Could not open ~/.crossfire/keys, trying to load global bindings");
 	if (client_libdir==NULL) {
 	    init_default_keybindings();
 	    return;
@@ -340,6 +368,7 @@ void init_keys()
     }
     while (fgets(buf, BIG_BUF, fp)) {
 	line++;
+    buf[BIG_BUF-1]='\0';
 	parse_keybind_line(buf,line,0);
     }
     fclose(fp);
@@ -478,6 +507,9 @@ static void parse_key(char key, uint32 keysym)
           (cpl.fire_on? "Fire&": ""),
           (cpl.run_on ? "Run&" : ""),
           keysym==NoSymbol? "unknown": gdk_keyval_name(keysym));
+#ifdef WIN32
+       if ( ( 65513 != keysym ) && ( 65511 != keysym ) )
+#endif
     draw_info(buf,NDI_BLACK);
     cpl.count=0;
 }
@@ -490,7 +522,11 @@ static void parse_key(char key, uint32 keysym)
  */
 static char * get_key_info(Key_Entry *key, int save_mode)
 {
-    static char buf[MAX_BUF];
+    /* bind buf is the maximum space allowed for a
+     * binded command. We will add additional datas to
+     * it so we increase by MAX_BUF*/
+    static char buf[MAX_BUF+sizeof(bind_buf)];
+
     char buff[MAX_BUF];
     int bi=0;
 
@@ -590,7 +626,7 @@ static void show_keys(int allbindings)
 
 void bind_key(char *params)
 {
-    char buf[MAX_BUF];
+    char buf[MAX_BUF + 16];
 
     if (!params) {
 	draw_info("Usage: bind [-nfre] {<commandline>/commandkey/firekey{1/2}/runkey{1/2}/",NDI_BLACK);
@@ -610,7 +646,7 @@ void bind_key(char *params)
 
     if (!strcmp(params, "firekey1")) {
 	bind_keysym = & firekeysym[0];
-	draw_info("Push key to bind new firekey 1.",NDI_BLACK);
+    draw_info("Push key to bind new firekey 1.",NDI_BLACK);
 	cpl.input_state = Configure_Keys;
 	return;
     }
@@ -694,8 +730,14 @@ void bind_key(char *params)
 	return;
     }
 
+    if (strlen(params) >= sizeof(bind_buf)) {
+	params[sizeof(bind_buf) - 1] = '\0';
+    draw_info("Keybinding too long! Truncated:",NDI_RED);
+    draw_info(params,NDI_RED);
+    }
     sprintf(buf, "Push key to bind '%s'.", params);
     draw_info(buf,NDI_BLACK);
+
     strcpy(bind_buf, params);
     cpl.input_state = Configure_Keys;
     return;
@@ -722,9 +764,18 @@ static void save_keys()
     int i;
     FILE *fp;
 
+#ifdef WIN32
+    /* Use player's name if available */
+    if ( strlen( cpl.name ) )
+        sprintf( buf,"%s/.crossfire/%s.keys", getenv("HOME"), cpl.name );
+    else
+        sprintf( buf,"%s/.crossfire/keys", getenv("HOME") );
+#else
     sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
+#endif
+
     if (make_path_to_file(buf)==-1) {
-	fprintf(stderr,"Could not create %s\n", buf);
+	LOG(LOG_WARNING,"gtk::save_keys","Could not create %s", buf);
 	return;
     }
     if ((fp=fopen(buf,"w"))==NULL) {
@@ -767,7 +818,7 @@ static void save_keys()
     }
 
     for (i=0; i<KEYHASH; i++) {
-	save_individual_key(fp, keys[i], 0);
+    save_individual_key(fp, keys[i], 0);
     }
     fclose(fp);
     /* Should probably check return value on all writes to be sure, but... */
@@ -807,14 +858,15 @@ static void configure_keys(uint32 keysym)
     }
 
     if (bind_keysym!=NULL) {
-	*bind_keysym=keysym;
+        *bind_keysym=keysym;
+        bind_keysym=NULL;
     }
     else {
 	insert_key(keysym, bind_flags, bind_buf);
     }
 
-    sprintf(buf, "Binded to key '%s' (%i)", 
-	  keysym==NoSymbol?"unknown":gdk_keyval_name(keysym), 0);
+    sprintf(buf, "Binded to key '%s' (%i)",
+	  keysym==NoSymbol?"unknown":gdk_keyval_name(keysym), keysym);
     draw_info(buf,NDI_BLACK);
     cpl.fire_on=0;
     cpl.run_on=0;
@@ -886,7 +938,7 @@ void unbind_key(char *params)
 			goto unbinded;
 		    }
 		}
-		fprintf(stderr,"unbind_key - found number entry, but could not find actual key\n");
+		LOG(LOG_ERROR,"gtk::unbind_key","found number entry, but could not find actual key");
 	    }
 	}
     }
@@ -947,6 +999,10 @@ void keyfunc(GtkWidget *widget, GdkEventKey *event, GtkWidget *window) {
 	if (event->keyval == completekeysym) gtk_complete_command();
 	if (event->keyval == prevkeysym || event->keyval == nextkeysym) 
 	    gtk_command_history(event->keyval==nextkeysym?0:1);
+#ifdef WIN32
+    else
+        gtk_widget_event(GTK_WIDGET(entrytext), (GdkEvent*)event);
+#endif
     } else {
 	switch(cpl.input_state) {
 	    case Playing:
@@ -1016,7 +1072,7 @@ void keyfunc(GtkWidget *widget, GdkEventKey *event, GtkWidget *window) {
 		break;
 
 	    default:
-		fprintf(stderr,"Unknown input state: %d\n", cpl.input_state);
+		LOG(LOG_ERROR,"gtk::keyfunc","Unknown input state: %d", cpl.input_state);
 	}
       
     }
