@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_map_c =
- *   "$Id: map.c,v 1.16 2003/08/08 17:06:40 tchize Exp $";
- */
+char *rcsid_gtk_map_c =
+    "$Id: map.c,v 1.23 2004/01/30 18:43:34 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -32,12 +30,19 @@
 #include "config.h"
 #include <stdlib.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 #include <png.h>
 
 /* Pick up the gtk headers we need */
 #include <gtk/gtk.h>
+#ifndef WIN32
 #include <gdk/gdkx.h>
+#else
+#include <time.h>
+#include <gdk/gdkwin32.h>
+#endif
 #include <gdk/gdkkeysyms.h>
 
 #include "client-types.h"
@@ -135,7 +140,7 @@ void reset_map()
     return;
 }
 
-void print_darkness()
+void print_darkness()/*too complicate to pass thru log*/
 {
 
     int x= 0;
@@ -151,7 +156,7 @@ void print_darkness()
     }
 }
 
-void print_map()
+void print_map()/*too complicate to pass thru log*/
 {
     int x= 0;
     int y= 0;
@@ -220,7 +225,7 @@ static int need_recenter_map( int dx, int dy)
 {
     
     if( pl_pos.x + dx + use_config[CONFIG_MAPWIDTH] +2 >= the_map.x ||
-	pl_pos.y + dx + use_config[CONFIG_MAPHEIGHT] +2 >= the_map.y ||
+	pl_pos.y + dy + use_config[CONFIG_MAPHEIGHT] +2 >= the_map.y ||
 	pl_pos.x + dx -2 <= 0                ||
 	pl_pos.y + dy -2 <= 0 )
     {
@@ -332,9 +337,9 @@ void display_mapscroll(int dx,int dy)
      * virtual position though
      */
 	
-    if( need_recenter_map( dx, dy) == TRUE) 
+    if( need_recenter_map( dx, dy) == TRUE)
 	recenter_virtual_map_view( &the_map);
-	
+
     pl_pos.x+= dx;
     pl_pos.y+= dy;
     local_mapx= pl_pos.x + use_config[CONFIG_MAPWIDTH];
@@ -350,21 +355,15 @@ void display_mapscroll(int dx,int dy)
     for( x= pl_pos.x; x < pl_pos.x + use_config[CONFIG_MAPWIDTH]; x++) {
 	for( y= pl_pos.y; y < pl_pos.y + use_config[CONFIG_MAPHEIGHT]; y++) {
 	    if( (x + dx) < pl_pos.x || (x + dx) >= (use_config[CONFIG_MAPWIDTH] + pl_pos.x) ||
-	       (y + dy) < pl_pos.y || (y + dy) >= (use_config[CONFIG_MAPHEIGHT] + pl_pos.y) ) 
+	       (y + dy) < pl_pos.y || (y + dy) >= (use_config[CONFIG_MAPHEIGHT] + pl_pos.y) )
 	    {
 		if( x < 0 || y < 0 || x >= the_map.x ||	y >= the_map.y)
 		    continue;
 
 
 		the_map.cells[x][y].need_update= 1;
-        /*After long long long investigations on why the multipart objects did
-          disappear when entering map view from right or bottom, scrolling
-          through whole server code, i concluded the following line should be
-          commented. If a multipart object was on a square outside of map,
-          containing the tail, tail may be cleared by following process and
-          so we end up with things like tower disappearance.
-                    tchize@myrealbox.com*/
-		/*the_map.cells[x][y].cleared= 1;*/
+		the_map.cells[x][y].cleared= 1;
+        the_map.cells[x][y].keephead = 1;/*otherwise we will also mark the multipart as cleared*/
 	    }
 	} /* for y */
     } /* for x */
@@ -432,7 +431,8 @@ void drawsmooth (int mx,int my,int layer,int picx,int picy){
     int emx,emy;
     int smoothface;
     int dosmooth=0;
-    if (the_map.cells[mx][my].heads[0].face==0)
+    if ( (the_map.cells[mx][my].heads[0].face==0)
+         || !CAN_SMOOTH(the_map.cells[mx][my].smooth[layer]) )
         return;
     for (i=0;i<8;i++){
         emx=mx+dx[i];
@@ -441,7 +441,7 @@ void drawsmooth (int mx,int my,int layer,int picx,int picy){
             slevels[i]=0;
             sfaces[i]=0; /*black picture*/
         }
-        if (the_map.cells[emx][emy].smooth[layer]<=the_map.cells[mx][my].smooth[layer]){            
+        if (the_map.cells[emx][emy].smooth[layer]<=the_map.cells[mx][my].smooth[layer]){
             slevels[i]=0;
             sfaces[i]=0; /*black picture*/
         }else{      
@@ -603,7 +603,7 @@ void gtk_draw_map(int redraw) {
               is nothing to draw at that layer (but there was something at lower
               layers). This is handled here. The else part is to take into account
               cases where the smooth as already been handled 2 code lines before*/
-            else if ( use_config[CONFIG_SMOOTH] && 
+            else if ( use_config[CONFIG_SMOOTH] &&
                  the_map.cells[mx][my].need_resmooth )
                 drawsmooth (mx,my,layer,x * map_image_size,y * map_image_size);
             
@@ -677,7 +677,7 @@ void gtk_draw_map(int redraw) {
 	/* I care about performance for 'long' updates, so put the check in to make
 	 * these a little more noticable */
 	if ((elapsed1 + elapsed2)>10000)
-	    fprintf(stderr,"sdl_gen_map: gen took %7ld, flip took %7ld, total = %7ld\n",
+	    LOG(LOG_INFO,"gtk::sdl_gen_map","gen took %7ld, flip took %7ld, total = %7ld",
 		    elapsed1, elapsed2, elapsed1 + elapsed2);
     }
 }
