@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_png_c =
- *   "$Id: png.c,v 1.6 2003/06/19 12:23:40 tchize Exp $";
- */
+char *rcsid_gtk_png_c =
+    "$Id: png.c,v 1.9 2004/01/30 18:43:34 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -28,14 +26,20 @@
 #include "config.h"
 #include <stdlib.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 #include <png.h>
-
 #include <client-types.h>
+#include <client.h>
 
 /* Pick up the gtk headers we need */
 #include <gtk/gtk.h>
+#ifndef WIN32
 #include <gdk/gdkx.h>
+#else
+#include <gdk/gdkwin32.h>
+#endif
 #include <gdk/gdkkeysyms.h>
 
 
@@ -152,7 +156,7 @@ uint8 *png_to_data(uint8 *data, int len, uint32 *width, uint32 *height)
 
     if (!pixels) {
 	png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
-	fprintf(stderr,"Out of memory - exiting\n");
+	LOG(LOG_CRITICAL,"gtk::png_to_data","Out of memory - exiting");
 	exit(1);
     }
 
@@ -232,7 +236,7 @@ uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale)
     if (*width > MAX_IMAGE_WIDTH || new_width > MAX_IMAGE_WIDTH
     || *height > MAX_IMAGE_HEIGHT || new_height > MAX_IMAGE_HEIGHT)
     {
-	fprintf(stderr, "Image too big\n");
+	LOG(LOG_CRITICAL,"gtk::rescale_rgba_data","Image too big");
 	exit(0);
     }
 
