@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_sound_c =
- *   "$Id: sound.c,v 1.5 2002/07/26 05:39:10 mwedel Exp $";
- */
+char *rcsid_gtk_sound_c =
+    "$Id: sound.c,v 1.10 2004/01/30 22:03:37 ryo_saeba Exp $";
 
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -38,6 +36,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <signal.h>
+#include <errno.h>
 #include <client-types.h>
 #include "client.h"
 
@@ -50,13 +49,13 @@ void signal_pipe(int i) {
 }
 
 FILE *sound_pipe=NULL;
-
-
+ChildProcess* sound_process;
 /* init_sounds open the audio device, and reads any configuration files
  * that need to be.  It returns 0 on success.  On failure, the calling
  * function will likely disable sound support/requests from the server.
  */
 
+#ifndef WIN32
 int init_sounds()
 {
     /* Easy trick - global nosound is set in the arg processing - if set,
@@ -64,29 +63,21 @@ int init_sounds()
      * the value of init_sounds, and not worry about checking nosound.
      */
     if (!want_config[CONFIG_SOUND]) return -1;
-    
-    if (sound_pipe) fclose(sound_pipe);
 
-    sound_pipe=popen(BINDIR "/cfsndserv","w");
-    /* if its not in its proper place, let the users shell
-     * try to find it - it may have a better idea.
-     */
-    if (!sound_pipe)
-	sound_pipe=popen("cfsndserv","w");
-
-    if (!sound_pipe){
-      perror("cfsndserver");
-      return -1;
-    }
-    signal(SIGPIPE, signal_pipe);
+    //if (sound_process) //kill
 
+    sound_process=raiseChild(BINDIR "/cfsndserv",CHILD_STDIN|CHILD_STDOUT|CHILD_STDERR);
+    logChildPipe(sound_process, LOG_INFO, CHILD_STDOUT|CHILD_STDERR);
+    sound_pipe=fdopen(sound_process->tube[0],"w");
+    signal(SIGPIPE, signal_pipe);/*perhaps throwing this out :\*/
     return 0;
 }
+#endif
 
 
 /* Plays sound 'soundnum'.  soundtype is 0 for normal sounds, 1 for
  * spell_sounds.  This might get extended in the future.  x,y are offset
- * (assumed from player) to play sound.  This information is used to 
+ * (assumed from player) to play sound.  This information is used to
  * determine value and left vs right speaker balance.
  *
  * This procedure seems to be very slow - much slower than I would
@@ -95,27 +86,28 @@ int init_sounds()
 
 static void play_sound(int soundnum, int soundtype, int x, int y)
 {
-    if (!use_config[CONFIG_SOUND]) return;
+#ifndef WIN32
 
-    if (fprintf(sound_pipe,"%4x %4x %4x %4x\n",soundnum,soundtype,x,y)<=0) {
-	use_config[CONFIG_SOUND]=0;
-	fclose(sound_pipe);
-	return;
+    if (!use_config[CONFIG_SOUND]) return;
+    if ( (fprintf(sound_pipe,"%4x %4x %4x %4x\n",soundnum,soundtype,x,y)<=0) ||
+         (fflush(sound_pipe)!=0) ){
+        LOG(LOG_ERROR,"gtk::play_sound","couldn't write to sound pipe: %d",errno);
+        use_config[CONFIG_SOUND]=0;
+        fclose(sound_pipe);
+        sound_process=NULL;
+        return;
     }
-    if (fflush(sound_pipe)!=0) {
-	use_config[CONFIG_SOUND]=0;
-	fclose(sound_pipe);
-	return;
-   }
+#endif
 }
 
 
+#ifndef WIN32
 void SoundCmd(unsigned char *data,  int len)
-{  
+{
     int x, y, num, type;
 
     if (len!=5) {
-	fprintf(stderr,"Got invalid length on sound command: %d\n", len);
+	LOG(LOG_WARNING,"gtk::SoundCmd","Got invalid length on sound command: %d", len);
 	return;
     }
     x = data[0];
@@ -129,4 +121,4 @@ void SoundCmd(unsigned char *data,  int len)
 #endif
     play_sound(num, type, x, y);
 }
-
+#endif
