@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_png_c =
- *   "$Id: png.c,v 1.10 2003/10/06 07:04:01 mwedel Exp $";
- */
+char *rcsid_x11_png_c =
+    "$Id: png.c,v 1.13 2004/04/19 05:14:11 mwedel Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -782,7 +780,7 @@ int rgba_to_xpixmap(Display *display, Drawable draw, uint8 *pixels,
 		   unsigned long width, unsigned long height)
 {
     int red,green,blue, lastred=-1, lastgreen=-1, lastblue=-1,alpha,x,y,
-	cmask, lastcmask, lastcolor;
+	cmask=-1, lastcmask, lastcolor=-1;
     GC	gc, gc_alpha;
 
     *pix = XCreatePixmap(display, draw, width, height, 
@@ -877,12 +875,14 @@ int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *r
 void get_map_image_size(int face, uint8 *w, uint8 *h)
 {
     /* This function is not implemented yet, so just return default values */
-    if (face == 0) {
+    if (face == 0 || pixmaps[face] == NULL) {
 	*w = 1;
 	*h = 1;
     }
-    *w = pixmaps[face]->width;
-    *h = pixmaps[face]->height;
+    else {
+	*w = pixmaps[face]->width;
+	*h = pixmaps[face]->height;
+    }
 }
 
 
