@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_x11_h =
- *   "$Id: x11.h,v 1.8 2002/05/22 04:58:26 mwedel Exp $";
+ *   "$Id: x11.h,v 1.9 2003/10/06 07:04:02 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -39,8 +39,7 @@ extern char *facetoname[MAXPIXMAPNUM];
 
 struct PixmapInfo {
   Pixmap pixmap,mask;
-/*  Pixmap bitmap;*/
-/*  long fg,bg;*/
+  uint8 width, height;
 };
 
 extern struct PixmapInfo *pixmaps[MAXPIXMAPNUM];
