@@ -6,6 +6,7 @@ extern int init_pngx_loader(Display *display);
 extern int png_to_xpixmap(Display *display, Drawable draw, unsigned char *data, int len, Pixmap *pix, Pixmap *mask, Colormap *cmap, unsigned long *width, unsigned long *height);
 extern int rgba_to_xpixmap(Display *display, Drawable draw, uint8 *pixels, Pixmap *pix, Pixmap *mask, Colormap *cmap, unsigned long width, unsigned long height);
 extern int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *rgba_data, int width, int height);
+extern void get_map_image_size(int face, uint8 *w, uint8 *h);
 /* sound.c */
 extern void signal_pipe(int i);
 extern int init_sounds(void);
@@ -28,6 +29,7 @@ extern void set_weight_limit(uint32 wlim);
 extern void set_scroll(char *s);
 extern void set_autorepeat(char *s);
 extern int get_info_width(void);
+extern void menu_clear(void);
 extern char *get_metaserver(void);
 extern void check_x_events(void);
 extern int init_windows(int argc, char **argv);
@@ -65,6 +67,5 @@ extern void reset_map(void);
 extern void print_darkness(void);
 extern void print_map(void);
 extern void set_map_darkness(int x, int y, uint8 darkness);
-extern void display_map_addbelow(long x, long y, long face);
 extern void display_mapscroll(int dx, int dy);
 extern void reset_map_data(void);
