@@ -1,7 +1,5 @@
-/*
- * static char *rcsid_xutil_c =
- *   "$Id: xutil.c,v 1.10 2003/10/06 07:04:02 mwedel Exp $";
- */
+char *rcsid_x11_xutil_c =
+    "$Id: xutil.c,v 1.16 2004/01/31 20:13:11 tchize Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -24,7 +22,7 @@
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* This contains varous 'support' functions.  These functions will probably
+/* This contains various 'support' functions.  These functions will probably
  * go mostly unaltered between different toolkits, as long as X11 is still
  * used. 
  */
@@ -247,7 +245,7 @@ static void insert_key(KeySym keysym, KeyCode keycode, int flags, char *command)
     int i, direction=-1;
 
     if (keycode>MAX_KEYCODE) {
-	fprintf(stderr,"Warning insert_key:keycode that is passed is greater than 255.\n");
+	LOG(LOG_WARNING,"x11::insert_key", "keycode that is passed is greater than 255.");
 	keycode=0;	/* hopefully the rest of the data is OK */
     }
     if (keys[keycode]==NULL) {
@@ -302,7 +300,7 @@ void parse_keybind_line(char *buf, int line, int standard)
 
     if (buf[0]=='#' || buf[0]=='\n') return;
     if ((cpnext = strchr(buf,' '))==NULL) {
-	fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line,buf);
+	LOG(LOG_WARNING,"x11::parse_keybind_line","Line %d (%s) corrupted.", line,buf);
 	return;
     }
     /* Special keybinding line */
@@ -311,13 +309,13 @@ void parse_keybind_line(char *buf, int line, int standard)
 	while (*cpnext == ' ') ++cpnext;
 	cp = strchr(cpnext, ' ');
 	if (!cp) {
-	    fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line,buf);
+	    LOG(LOG_WARNING,"x11::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line,buf);
 	    return;
 	}
 	*cp++ = 0;  /* Null terminate it */
 	cp1 = strchr(cp, ' ');
 	if (!cp1) {
-	    fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line,buf);
+	    LOG(LOG_WARNING,"x11::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line,buf);
 	    return;
 	}
 	*cp1 ++ = 0;/* Null terminate it */
@@ -325,7 +323,7 @@ void parse_keybind_line(char *buf, int line, int standard)
 	keysym = XStringToKeysym(cp);
 	/* As of now, all these keys must have keysyms */
 	if (keysym == NoSymbol) {
-	    fprintf(stderr,"Could not convert %s into keysym\n", cp);
+	    LOG(LOG_WARNING,"x11::parse_keybind_line","Could not convert %s into keysym", cp);
 	    return;
 	}
 	if (!strcmp(cpnext,"commandkey")) {
@@ -376,7 +374,7 @@ void parse_keybind_line(char *buf, int line, int standard)
     keysym = XStringToKeysym(buf);
     cp = cpnext;
     if ((cpnext = strchr(cp,' '))==NULL) {
-	fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line, cp);
+	LOG(LOG_WARNING,"x11::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line, cp);
 	return;
     }
     *cpnext++ = '\0';
@@ -385,7 +383,7 @@ void parse_keybind_line(char *buf, int line, int standard)
     keycode = atoi(cp);
     cp = cpnext;
     if ((cpnext = strchr(cp,' '))==NULL) {
-	fprintf(stderr,"Line %d (%s) corrupted in keybinding file.\n", line, cp);
+	LOG(LOG_WARNING,"x11::parse_keybind_line","Line %d (%s) corrupted in keybinding file.", line, cp);
 	return;
     }
     *cpnext++ = '\0';
@@ -411,7 +409,7 @@ void parse_keybind_line(char *buf, int line, int standard)
 		flags |= KEYF_STANDARD;
 		break;
 	default:
-	    fprintf(stderr,"Warning:  Unknown flag (%c) line %d in key binding file\n",
+	    LOG(LOG_WARNING,"x11::parse_keybind_line","Unknown flag (%c) line %d in key binding file",
 		*cp, line);
         }
         cp++;
@@ -434,12 +432,16 @@ void parse_keybind_line(char *buf, int line, int standard)
          * it away, but at least print a warning message.
          */
         if (keycode==0) {
-	    fprintf(stderr,"Warning: could not convert keysym %s into keycode, ignoring\n",
+	    LOG(LOG_WARNING,"x11::parse_keybind_line","could not convert keysym %s into keycode, ignoring",
 		buf);
 	}
     }
     /* Rest of the line is the actual command.  Lets kill the newline */
     cpnext[strlen(cpnext)-1]='\0';
+    if (strlen(cpnext)>(sizeof(bind_buf)-1)){
+        cpnext[sizeof(bind_buf)-1]='\0';
+        LOG(LOG_WARNING,"gtk::parse_keybind_line","Had to truncate a too long command");
+    }
     insert_key(keysym, keycode, flags | standard, cpnext);
 }
 
@@ -516,7 +518,7 @@ void init_keys()
 
     sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
     if ((fp=fopen(buf,"r"))==NULL) {
-	fprintf(stderr,"Could not open ~/.crossfire/keys, trying to load global bindings\n");
+	LOG(LOG_INFO,"x11::init_keys","Could not open ~/.crossfire/keys, trying to load global bindings");
 	if (client_libdir==NULL) {
 	    init_default_keybindings();
 	    return;
@@ -703,7 +705,10 @@ void parse_key(char key, KeyCode keycode, KeySym keysym, int repeated)
  */
 static char * get_key_info(Key_Entry *key, KeyCode kc, int save_mode)
 {
-    static char buf[MAX_BUF];
+    /* bind buf is the maximum space allowed for a
+     * binded command. We will add additional datas to
+     * it so we increase by MAX_BUF*/
+    static char buf[MAX_BUF+sizeof(bind_buf)];
     char buff[MAX_BUF];
     int bi=0;
 
@@ -917,6 +922,11 @@ void bind_key(char *params)
     return;
   }
 
+    if (strlen(params) >= sizeof(bind_buf)) {
+	params[sizeof(bind_buf) - 1] = '\0';
+    draw_info("Keybinding too long! Truncated:",NDI_RED);
+    draw_info(params,NDI_RED);
+    }
   sprintf(buf, "Push key to bind '%s'.", params);
   draw_info(buf,NDI_BLACK);
   strcpy(bind_buf, params);
@@ -948,7 +958,7 @@ static void save_keys()
 
     sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
     if (make_path_to_file(buf)==-1) {
-	fprintf(stderr,"Could not create %s\n", buf);
+	LOG(LOG_WARNING,"x11::save_keys","Could not create %s", buf);
 	return;
     }
     if ((fp=fopen(buf,"w"))==NULL) {
@@ -1107,7 +1117,7 @@ void unbind_key(char *params)
 			goto unbinded;
 		    }
 		}
-		fprintf(stderr,"unbind_key - found number entry, but could not find actual key\n");
+		LOG(LOG_ERROR,"x11::unbind_key","found number entry, but could not find actual key");
 	    }
 	}
     }
@@ -1472,6 +1482,7 @@ void display_mapscroll(int dx,int dy)
 
 		the_map.cells[x][y].cleared= 1;
 		the_map.cells[x][y].need_update= 1;
+        the_map.cells[x][y].keephead = 1;/*otherwise we will also mark the multipart as cleared*/
 	    }
 	} /* for y */
     } /* for x */
